# BuckoFIBS.nsi - The script to NSIS, the NullSoft Install System.
# $Id: BuckoFIBS.nsi,v 1.5 2010/03/03 12:19:49 inim Exp $
#              Copyright© 2009 - Dick Balaska - BuckoSoft, Corp.
#              Released under GNU General Public License Version 3
# The NullSoft Install System can be found here http://www.nullsoft.com/free/nsis/

; The name of the installer
Name "BuckoFIBS"
;Icon "choo.ico"

Var	JAVA_HOME
Var JAVA_BIN
Var JAVA_INSTALLATION_MSG
Var	JAVA_VER
;Var	BF_JAR

; The file to write
;OutFile "BuckoFIBS-0.3.exe"
!include outfilename.txt

; The default installation directory
InstallDir $PROGRAMFILES\BuckoFIBS

; Registry key to check for directory (so if you install again, it will 
; overwrite the old one automatically)
InstallDirRegKey HKLM "Software\BuckoFIBS" "Install_Dir"

; Request application privileges for Windows Vista
RequestExecutionLevel admin

;--------------------------------

; Pages

Page components
Page directory
Page instfiles

UninstPage uninstConfirm
UninstPage instfiles

;--------------------------------

; The stuff to install
Section "BuckoFIBS (required)"

	Call LocateJava
	StrCmp "" $JAVA_INSTALLATION_MSG Success
	Abort $JAVA_INSTALLATION_MSG
	Goto Done
	
	Success:
		SectionIn RO
  
		; Set output path to the installation directory.
		SetOutPath $INSTDIR
  
		; Put file there
		File "[JARFILE]"
		File "choo.ico"
  
		CreateDirectory "$PROFILE\BuckoFIBS\db"
		; Write the installation path into the registry
		WriteRegStr HKLM SOFTWARE\BuckoFIBS "Install_Dir" "$INSTDIR"
  
		; Write the uninstall keys for Windows
		WriteRegStr HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\BuckoFIBS" "DisplayName" "BuckoFIBS [VERSION]"
		WriteRegStr HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\BuckoFIBS" "Publisher" "BuckoSoft, Corp."
		WriteRegStr HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\BuckoFIBS" "DisplayVersion" "6.9"
		WriteRegStr HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\BuckoFIBS" "UninstallString" '"$INSTDIR\uninstall.exe"'
		WriteRegDWORD HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\BuckoFIBS" "NoModify" 1
		WriteRegDWORD HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\BuckoFIBS" "NoRepair" 1
		WriteUninstaller "uninstall.exe"

	Done:
SectionEnd

; Optional section (can be disabled by the user)
Section "Start Menu Shortcuts"
  StrCpy $0 '-jar "$INSTDIR\[JARFILE]"'
  CreateDirectory "$SMPROGRAMS\BuckoFIBS"
  CreateShortCut "$SMPROGRAMS\BuckoFIBS\Uninstall.lnk" "$INSTDIR\uninstall.exe" "" "$INSTDIR\uninstall.exe" 0
  SetOutPath "$PROFILE\BuckoFIBS"
  CreateShortCut "$SMPROGRAMS\BuckoFIBS\BuckoFIBS.lnk" "$JAVA_BIN" '$0' "$INSTDIR\choo.ico" 0
  CreateShortCut "$DESKTOP\BuckoFIBS.lnk" "$JAVA_BIN" '$0' "$INSTDIR\choo.ico" 0
  
SectionEnd

Function LocateJava
	;Check for Java version and location
	Push $0
	Push $1
 
	ReadRegStr $JAVA_VER HKLM "SOFTWARE\JavaSoft\Java Runtime Environment" CurrentVersion
	StrCmp "" "$JAVA_VER" JavaNotPresent CheckJavaVer
 
	JavaNotPresent:
		StrCpy $JAVA_INSTALLATION_MSG "Java Runtime Environment is not installed on your computer. You need version 1.4 or newer to run this program."
		Goto Done
 
	CheckJavaVer:
		ReadRegStr $0 HKLM "SOFTWARE\JavaSoft\Java Runtime Environment\$JAVA_VER" JavaHome
		GetFullPathName /SHORT $JAVA_HOME "$0"
		StrCpy $JAVA_BIN "$JAVA_HOME\bin\java.exe"
		StrCpy $0 $JAVA_VER 1 0
		StrCpy $1 $JAVA_VER 1 2
		StrCpy $JAVA_VER "$0$1"
		IntCmp 16 $JAVA_VER FoundCorrectJavaVer FoundCorrectJavaVer JavaVerNotCorrect
 
	FoundCorrectJavaVer:
		IfFileExists "$JAVA_HOME\bin\javaw.exe" 0 JavaNotPresent
		;MessageBox MB_OK "Found Java: $JAVA_VER at $JAVA_HOME"
		Goto Done
 
	JavaVerNotCorrect:
		StrCpy $JAVA_INSTALLATION_MSG "The version of Java Runtime Environment installed on your computer is $JAVA_VER. Version 1.6 or newer is required to run this program."
 
	Done:
		Pop $1
		Pop $0
FunctionEnd

;--------------------------------

; Uninstaller

Section "Uninstall"
  
  ; Remove registry keys
  DeleteRegKey HKLM "Software\Microsoft\Windows\CurrentVersion\Uninstall\BuckoFIBS"
  DeleteRegKey HKLM SOFTWARE\BuckoFIBS

  ; Remove files and uninstaller
  Delete $INSTDIR\[JARFILE]
  Delete $INSTDIR\uninstall.exe

  ; Remove shortcuts, if any
  Delete "$SMPROGRAMS\BuckoFIBS\*.*"

  ; Remove directories used
  RMDir "$SMPROGRAMS\BuckoFIBS"
  RMDir "$INSTDIR"

SectionEnd
