/******************************************************************************
 * ClientReceiveParser.java - Parse messages received from the network.
 * $Id: ClientReceiveParser.java,v 1.6 2010/03/03 13:12:21 inim Exp $
 * 
 * BuckoFIBS - Backgammon by BuckoSoft
 * Copyright© 2009,2010 - Dick Balaska - BuckoSoft, Corp.
 * 
 * $Log: ClientReceiveParser.java,v $
 * Revision 1.6  2010/03/03 13:12:21  inim
 * Replaced (c) sign in comment mangled by CVS default encoding back to UTF-8
 *
 * Revision 1.5  2010/03/03 12:19:49  inim
 * Moved source to UTF8 encoding from CP1252 encoding. To this end all source files' (c) message was updated to "Copyright© 2009,2010 - Dick Balaska - BuckoSoft, Corp.". This replaces the (c) sign to UTF8, and adds the new year 2010.
 *
 * Revision 1.4  2010/02/15 09:22:22  dick
 * Play audio cues when Your opponent doubles or when he drops the match.
 *
 * Revision 1.3  2010/02/09 02:57:39  dick
 * Don't do autoGreedyBearoff if only watching a match.
 *
 * Revision 1.2  2010/02/08 09:14:13  dick
 * Add support for autoGreedyBearoff.  When the game turns to a race, enable greedy bearoffs.
 *
 * Revision 1.1  2010/02/04 05:57:53  inim
 * Mavenized project folder layout
 *
 * Revision 1.60  2010/01/25 21:09:58  dick
 * Remove the savedMatch icon from the playerList when the match is done.
 * Set the chatPane to "Tell (lastOpponent)".
 *
 * Revision 1.59  2010/01/23 06:14:35  dick
 * Handle removedFinishedMatch().  Handle youInvited().
 *
 * Revision 1.58  2009/05/09 03:42:27  dick
 * Add PlayerBannedWatch and Doesn'tWantYouToWatch.
 *
 * Revision 1.57  2009/03/16 18:52:17  dick
 * Play the YourTurn sound only if you can actually move.  (Respond to the "Please move" message.)
 *
 * Revision 1.56  2009/03/04 19:06:26  dick
 * Do message routing for the PlayerReportPane.
 *
 * Revision 1.55  2009/02/27 05:47:41  dick
 * Handle the whois "last logout" line.
 *
 * Revision 1.54  2009/02/25 09:41:12  dick
 * Handle ATTENTION messages from FIBS when it is shutting down.
 *
 * Revision 1.53  2009/02/25 07:42:54  dick
 * Calculate the duration of the match and store that with the FinishedMatch, will make a nice statistic someday.
 *
 * Revision 1.52  2009/02/24 05:48:47  dick
 * Handle connectionAborted() and ERROR from the ClientAdapter.
 *
 * Revision 1.51  2009/02/22 07:05:47  dick
 * if props.isDisplayRecv() then display them all.
 *
 * Revision 1.50  2009/02/21 17:59:42  dick
 * Update the RatingGraphPanel when a match is complete.
 *
 * Revision 1.49  2009/02/20 10:21:23  dick
 * Fill out a FinishedMatch and send it to the database.
 *
 * Revision 1.48  2009/02/17 21:12:56  dick
 * Display unknown messages as an error, not as an unhandled cookie.
 * (We know that the cookie is "unknown").
 *
 * Revision 1.47  2009/02/16 06:57:07  dick
 * Dispatch to ROLL_OR_DOUBLE instead of calling the methods directly.
 *
 * Revision 1.46  2009/02/14 12:19:15  dick
 * Between move work.
 *
 * Revision 1.45  2009/02/12 06:25:06  dick
 * Update the board when resuming a match.
 *
 * Revision 1.44  2009/02/11 09:05:48  dick
 * Better beginning and ending your turn.
 *
 * Revision 1.43  2009/02/09 22:44:28  dick
 * shutdown() cancels pending timers.
 * Handle resign (both ways).
 *
 * Revision 1.42  2009/02/06 07:56:48  dick
 * Working on doubles.
 *
 * Revision 1.41  2009/02/03 09:44:18  dick
 * Just give us a handle to MainDialog already.
 * Add some audio events.
 *
 * Revision 1.40  2009/02/02 08:42:24  dick
 * Several various message cleanups.
 *
 * Revision 1.39  2009/02/01 23:22:59  dick
 * FIBS_Done is an in-game message.
 *
 * Revision 1.38  2009/02/01 21:15:06  dick
 * Fire off a "show saved" message after we have finished who'ing.
 *
 * Revision 1.37  2009/01/31 08:46:25  dick
 * Fire off a 'who' 1 second after we finish the initial table.  FIBS doesn't send all the players across.
 * YourMove goes to the GameManager now.
 *
 * Revision 1.36  2009/01/31 06:08:49  dick
 * Dispatch board and double commands to the GameManager.
 *
 * Revision 1.35  2009/01/29 08:27:05  dick
 * Dispose of "waves" commands gracefully.
 *
 * Revision 1.34  2009/01/28 22:28:27  dick
 * Try a different Javadoc reference to the Mode.
 *
 * Revision 1.33  2009/01/28 22:26:22  dick
 * Javadoc.
 *
 * Revision 1.32  2009/01/28 19:42:47  dick
 * Prettier cvs link in the javadoc.
 *
 * Revision 1.31  2009/01/28 08:34:18  dick
 * If we get a FIBS_Unknown, then display the previous message too, because it's probably
 * a busted end-of-string detector from the previous message.
 *
 * Revision 1.30  2009/01/27 19:14:31  dick
 * Handle savedMatch as a warning.
 *
 * Revision 1.29  2009/01/27 06:54:23  dick
 * uninvite us if a player starts playing.
 *
 * Revision 1.28  2009/01/27 05:44:43  dick
 * public parseMessage becomes private dispatchMessage because the entry point is now the Adapter's dispatch().
 *
 * Revision 1.27  2009/01/26 17:35:15  dick
 * Push the CookieMonster down to the ClientConnection.  ClientAdapter now emits the cookie with the string.
 * There are many messages besides the known "BadBoard" messages that are a result of runon messages (missing crlf).
 * CookieMonster is going to need to do a full regexp scan and see if there is anything on the right that needs to be pushed back as another message.
 *
 * Revision 1.26  2009/01/26 07:22:51  dick
 * Process the "You Roll" command as a yourMove trigger.
 *
 * Revision 1.25  2009/01/24 17:05:05  dick
 * Add parseReplayMessage(s).
 *
 * Revision 1.24  2009/01/18 04:51:26  dick
 * add dispatch(YOUR_MOVE);
 * CommandDispatcher becomes CommandDispatcherImpl.
 * I should probably abstract out the methods that are being directly called in the Impl.
 *
 * Revision 1.23  2009/01/13 18:27:39  dick
 * Handle Ready To Play
 *
 * Revision 1.22  2009/01/12 21:59:39  dick
 * Javadoc.
 *
 * Revision 1.21  2009/01/12 20:08:29  dick
 * Implement ClientAdapter as part of the new ClientConnection decoupling.
 *
 * Revision 1.20  2009/01/10 05:04:02  dick
 * Handle match over messages special.  Fibs doesn't send a final board after the match is over, so we have to fake it out.
 *
 * Revision 1.19  2009/01/09 19:09:32  dick
 * Handle the FIBS_Timeout message.
 * Handle more game messages.
 *
 * Revision 1.18  2009/01/09 08:10:02  dick
 * Chat, game, and system message routing.
 *
 * Revision 1.17  2009/01/07 02:32:45  dick
 * Better resetting of our state between client connections.
 *
 * Revision 1.16  2009/01/06 08:05:28  dick
 * Handle registering a new user.
 *
 * Revision 1.15  2008/12/20 23:02:10  dick
 * Support for Own Info.
 *
 * Revision 1.14  2008/12/15 01:47:36  dick
 * Just echo FirstRoll and MakesFirstMove
 *
 * Revision 1.13  2008/12/14 06:43:23  dick
 * Add YouAreWatching (switch to board).
 * Only display a Cookie? if the pref says to.
 *
 * Revision 1.12  2008/12/13 06:57:11  dick
 * Move ClientConnection, CookieMonster, and FIBSMessages to their own network package.
 *
 * Revision 1.11  2008/12/13 06:20:49  dick
 * Dispatch NetworkConnected when we receive the welcome message.
 *
 * Revision 1.10  2008/12/11 20:27:33  dick
 * Handle CantInviteSelf, StartingNewGame, and Board.
 *
 * Revision 1.9  2008/12/11 09:58:50  dick
 * Display Network In messages in a different color.
 *
 * Revision 1.8  2008/12/11 08:46:19  dick
 * parse incoming Chat Message.
 *
 * Revision 1.7  2008/12/10 17:59:25  dick
 * Display system messages in color.
 *
 * Revision 1.6  2008/12/09 19:32:36  dick
 * Add Goodbye and NewMatchRequest handling.
 *
 * Revision 1.5  2008/12/09 01:48:55  dick
 * Early CookieMonster integration.
 *
 * Revision 1.4  2008/12/07 22:52:27  dick
 * Start playing with the cookie monster.
 *
 * Revision 1.3  2008/04/05 01:42:11  dick
 * Add eol when outputting the motd.
 *
 * Revision 1.2  2008/04/01 04:21:26  dick
 * Handle player logging out.
 *
 * Revision 1.1  2008/03/31 07:08:00  dick
 * Parse messages received from the network.
 *
 */

/* 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * The Original Code is BuckoFIBS, <http://www.buckosoft.com/BuckoFIBS/>.
 * The Initial Developer of the Original Code is Dick Balaska and BuckoSoft, Corp.
 * 
 */
package com.buckosoft.fibs.BuckoFIBS;

import java.util.Date;
import java.util.Timer;
import java.util.TimerTask;

import com.buckosoft.fibs.BuckoFIBS.AudioManager.Cue;
import com.buckosoft.fibs.BuckoFIBS.gui.MainDialog;
import com.buckosoft.fibs.BuckoFIBS.gui.SystemMessagesTextPane;
import com.buckosoft.fibs.BuckoFIBS.gui.chatWindow.ChatPane;
import com.buckosoft.fibs.domain.FinishedMatch;
import com.buckosoft.fibs.domain.Line;
import com.buckosoft.fibs.domain.Player;
import com.buckosoft.fibs.net.ClientAdapter;
import com.buckosoft.fibs.net.ClientConnection;
import com.buckosoft.fibs.net.FIBSMessages;


/** Parse and dispatch messages received from the network.
 * @author Dick Balaska
 * @since 2008/03/31
 * @version $Revision: 1.6 $ <br> $Date: 2010/03/03 13:12:21 $
 * @see <a href="http://cvs.buckosoft.com/Projects/BuckoFIBS/BuckoFIBS/src/com/buckosoft/fibs/BuckoFIBS/ClientReceiveParser.java">cvs ClientReceiveParser.java</a>
 */
public class ClientReceiveParser implements FIBSMessages, ClientAdapter {
	private	final static boolean DEBUG = false;

	/** We connect to fibs in either normal communications mode, or a special "Register new user" mode.
	 */
	public enum Mode {
		/** Default, normal fibs connection */
		Run,
		/** Special register new user commands are needed. */
		Register
	}
	private	MainDialog		mainDialog;
	private	CommandDispatcher	commandDispatcher = null;
	private	BFProperties		properties = null;
	private ClientConnection clientConnection = null;
	private	Mode			mode = Mode.Run;
	private	final static String eol = "\r\n";

	private	String lastInviter;
	private	String lastMessage0;
	private	String lastMessage1;
	private int		lastCookie0;
	private int		lastCookie1;
	
	private	boolean	whoVirgin = true;
	private	boolean needSavedGames = true;
	private	Date	matchStartTime = null;
//	private	boolean	matchResuming = false;
	private	boolean	matchResumingYourTurn = false;

	private	FinishedMatch		finishedMatch = null;

	/** Set the reference to the instance of MainDialog
	 * @param mainDialog The MainDialog that is running
	 */
	public	void setMainDialog(MainDialog mainDialog) {
		this.mainDialog = mainDialog;
	}

	/** Set the reference to our CommandDispatcher
	 * @param commandDispatcher The CommandDispatcher
	 */
	public void setCommandDispatcher(CommandDispatcher commandDispatcher) {
		this.commandDispatcher = commandDispatcher;
	}

	/** Set the reference to our ClientConnection
	 * @param clientConnection The ClientConnection
	 */
	public void setClientConnection(ClientConnection clientConnection) {
		this.clientConnection = clientConnection;
	}

	/** Set the Properties/User preferences
	 * @param properties The loaded properties
	 */
	public	void setProperties(BFProperties properties) {
		this.properties = properties;
	}

	/** We connect to fibs in either normal communications mode, or a special "Register new user" mode.
	 * @param mode The mode that we should parse messages in
	 * @see com.buckosoft.fibs.BuckoFIBS.ClientReceiveParser.Mode
	 */
	public void setMode(Mode mode) {
		this.mode = mode;
		clientConnection.resetFIBSCookieMonster();
	}

	/** Kill all timers and tasks this object owns as we prepare to go bye-bye.
	 */
	public void shutdown() {
		if (this.myTimer != null) {
			this.myTimer.cancel();
			this.myTimer = null;
		}
		if (this.myTimerSavedGames != null) {
			this.myTimerSavedGames.cancel();
			this.myTimerSavedGames = null;
		}
		if (this.myTimerTaskSavedGames != null) {
			this.myTimerTaskSavedGames.cancel();
			this.myTimerTaskSavedGames = null;
		}
	}

	private void dispatchMessage(int cookie, String s) {
		lastMessage1 = lastMessage0;
		lastMessage0 = s;
		lastCookie1 = lastCookie0;
		lastCookie0 = cookie;
		if (DEBUG)
			System.out.println("cookie = " + cookie);
		if (this.properties.isDisplayRecv())
			this.commandDispatcher.writeSystemMessageln(SystemMessagesTextPane.NETWORKMSG, s);
		if (mode == Mode.Register) {
			parseMessageRegister(s, cookie);
			return;
		}
		switch (cookie) {
		// These do nothing here.
		case CLIP_MOTD_END:
		case FIBS_BoardstyleSetTo:
		case FIBS_SavedMatchesHeader:
		case FIBS_Empty:
		case FIBS_UsersHeardYou:	// JavaFIBS puts this in the status bar, which i don't have
		case FIBS_YouAlreadyRolled:
			break;
		case CLIP_OWN_INFO:
			this.commandDispatcher.dispatch(CommandDispatcher.Command.OWN_INFO, s);
			break;
		case CLIP_WHO_INFO:
			parseWhoInfo(s);
			break;
		case CLIP_WHO_END:
			handleWhoEnd();
			break;
		case FIBS_SavedMatchPlaying:
		case FIBS_SavedMatchReady:
		case FIBS_SavedMatch:
			//if (DEBUG)
				System.out.println(s);
			this.commandDispatcher.dispatch(CommandDispatcher.Command.SAVED_MATCH, s);
			break;
		case FIBS_LoginPrompt:
			whoVirgin = true;
			this.commandDispatcher.writeSystemMessageln(s);
			clientConnection.sendLogin();
			break;
		case CLIP_LOGIN:
			parsePlayerLoggedIn(s);
			break;
		case CLIP_LOGOUT:
			parsePlayerLoggedOut(s);
			break;
		case FIBS_OpponentLogsOut:
		case FIBS_OpponentLeftGame:
			parseAbortMatch(s);
			break;
		case FIBS_Goodbye:
			disconnectFromServer();
			break;
		case FIBS_Timeout:
			this.commandDispatcher.writeSystemMessageln(SystemMessagesTextPane.NETWORKMSG, s);
			disconnectFromServer();
			break;
		case CLIP_WELCOME:
			this.commandDispatcher.dispatch(CommandDispatcher.Command.NETWORK_CONNECTED);
			break;
		case FIBS_YouAreWatching:
			this.commandDispatcher.writeSystemMessageln(SystemMessagesTextPane.NETWORKMSG, s);
			this.commandDispatcher.dispatch(CommandDispatcher.Command.WATCHING);
			break;
		case CLIP_SAYS:
			if (s.startsWith("12 RepBotNG ")) {
				this.mainDialog.getPlayerReportPane().receiveLine(s, cookie);
				return;
			}
			parseChatMessage(s, cookie);
			break;
		case CLIP_WHISPERS:
		case CLIP_KIBITZES:
			parseChatMessage(s, cookie);
			break;
		case CLIP_SHOUTS:
			if (s.startsWith("13 MissManners"))
				this.commandDispatcher.dispatch(CommandDispatcher.Command.MISS_MANNERS, s);
			if(!properties.isMsgIgnoreShouts())
				parseChatMessage(s, cookie);
			break;
		case CLIP_YOU_SAY:
		case CLIP_YOU_SHOUT:
		case CLIP_YOU_WHISPER:
		case CLIP_YOU_KIBITZ:
			parseYouChatMessage(s, cookie);
			break;
		case FIBS_Waves:
		case FIBS_WavesAgain:
			if(!properties.isMsgIgnoreShouts())
				parseWavesMessage(s);
			break;
		case FIBS_NewMatchRequest:
		case FIBS_ResumeMatchRequest:
			parseInvite(s);
			break;
		case FIBS_CantInviteSelf:
		case FIBS_NotInteresting:
		case FIBS_PlayerNotPlaying:
		case FIBS_PlayerRefusingGames:
		case FIBS_NotYourTurnToRoll:
		case FIBS_NotYourTurnToMove:
		case FIBS_PlayerBannedWatch:
		case FIBS_DoesntWantYouToWatch:
			showErrorMessage(s);
			break;
		case FIBS_StartingNewGame:
			startNewGame();
			break;
		case FIBS_ResumeMatchAck0:
		case FIBS_ResumeMatchAck5:
			parseGameMessage(s);
			startNewGame();
//			this.matchResuming = true;
			break;
		case FIBS_Turn:
			parseGameMessage(s);
			parseResumeMatchTurn(s);
			break;
		case FIBS_MatchLength:
			parseGameMessage(s);
			parseResumeMatchLength(s);
			break;
		case FIBS_Board:
			if (DEBUG)
				System.out.println(s);
			Line gl = new Line(cookie, s);
			this.commandDispatcher.dispatch(CommandDispatcher.Command.GAME_MOVE, gl);
			if (this.matchResumingYourTurn && this.mainDialog.getBoard().isYouMayDouble()) {
				if (this.mainDialog.getBoard().getDice()[0][0] == 0) {
					this.commandDispatcher.dispatch(CommandDispatcher.Command.ROLL_OR_DOUBLE);
				}
				this.matchResumingYourTurn = false; 
			}
			doAutoGreedyBearoff();
			break;
		case FIBS_YouWinMatch:
		case FIBS_PlayerWinsMatch:
			parseMatchOverMessage(s, cookie);
			parseGameMessage(s);
			break;
		case FIBS_ReadyTrue:
		case FIBS_ReadyFalse:
			parseReadyMessage(cookie);
			break;
		case FIBS_AcceptRejectDouble:
			handleAcceptRejectDouble();
			break;
		case FIBS_JoinNextGame: 
			this.commandDispatcher.writeNetworkMessageln("join");
			break;
		case FIBS_FirstRoll:
		case FIBS_MakesFirstMove:
		case FIBS_PlayerRolls:
		case FIBS_PlayerMoves:
		case FIBS_CantMove:
		case FIBS_OnlyPossibleMove:
		case FIBS_PlayerWinsGame:
		case FIBS_ScoreUpdate:
		case FIBS_ResignWins:
		case FIBS_PlayerStartsWatching:
		case FIBS_WatchResign:
		case FIBS_ResignRefused:
		case FIBS_AcceptWins:
		case FIBS_PointsFor:
		case FIBS_YouWinGame:
		case FIBS_CantMoveFirstMove:
		case FIBS_RollBeforeMove:
		case FIBS_Done:
		case FIBS_GreedyTrue:			// XXX: Someday, change the icon on this state
		case FIBS_GreedyFalse:
		case FIBS_YouAcceptAndWin:
		case FIBS_YouResign:
		case FIBS_ResignYouWin:
		case FIBS_PleaseWaitForJoin:
		case FIBS_YouGiveUp:
		case FIBS_YouReject:
		case FIBS_MustMove:
			parseGameMessage(s);
			break;
		case FIBS_PlayerAcceptsDouble:
			this.mainDialog.getBoard().setYouDouble(false);
			parseGameMessage(s);
			break;
		case FIBS_YouCantMove:
			this.mainDialog.youCantMove();
			parseGameMessage(s);
			break;
		case FIBS_BearingOff:
			if (this.mainDialog.getBoard().isYourMove())
				this.commandDispatcher.dispatch(CommandDispatcher.Command.BEAR_OFF);
			parseGameMessage(s);
			break;
			
		case FIBS_PlayersStartingMatch:
		case FIBS_MatchResult:
		case FIBS_PlayerStopsWatching:
		case FIBS_PlayersStartingUnlimitedMatch:
		case FIBS_ResumingLimitedMatch:
		case FIBS_ResumingUnlimitedMatch:
			parseOtherMatchMessage(s);
			break;
		case FIBS_PlayerInfoStart:
		case FIBS_LastLogin:
		case FIBS_StillLoggedIn:
		case FIBS_NotLoggedIn:
		case FIBS_ReadyWatchingPlaying:
		case FIBS_RatingExperience:
		case FIBS_NoEmail:
		case FIBS_EmailAddress:
		case FIBS_IsPlayingWith:
		case FIBS_HasSavedGames:
		case FIBS_HasNoSavedGames:
		case FIBS_NoInfo:
			this.mainDialog.getPlayerReportPane().receiveLine(s, cookie);
//			this.commandDispatcher.writeSystemMessageln(SystemMessagesTextPane.NETWORKMSG, s);
			break;
		// Just echo the message to the window
		case FIBS_PreLogin:
		case CLIP_MOTD_BEGIN:
		case FIBS_TypeJoin:
		case FIBS_YouStopWatching:
		case FIBS_NewMatchAck2:
		case FIBS_YouAlreadyPlaying:
		case FIBS_AlreadyPlaying:
		case FIBS_LastLogout:
		case FIBS_NotPlaying:
		case FIBS_NoUser:
		case FIBS_EchoJunk:
			this.commandDispatcher.writeSystemMessageln(SystemMessagesTextPane.NETWORKMSG, s);
			break;
		case FIBS_DidntInvite:
		case FIBS_NotWatchingPlaying:
		case FIBS_NoSavedGames:
			this.commandDispatcher.writeSystemMessageln(SystemMessagesTextPane.NETWORKMSG, s);
			this.mainDialog.setSystemMessagesTabVisible();
			break;
		case FIBS_NewMatchAck10:
			this.commandDispatcher.writeSystemMessageln(SystemMessagesTextPane.NETWORKMSG, s);
			this.mainDialog.setGameMessagesTabVisible();
			break;
		case FIBS_WARNINGSavedMatch:
			parseInviteWarning(s);
			break;
		case FIBS_DoubleTrue:
			parseDoublesOnOff(true);
			parseGameMessage(s);
			break;
		case FIBS_DoubleFalse:
			parseDoublesOnOff(false);
			parseGameMessage(s);
			break;
		case FIBS_Doubles:
			parseDoubles(s);
			parseGameMessage(s);
			break;
		case FIBS_YouDouble:
			parseGameMessage(s);
			this.mainDialog.getBoard().setYouDouble(true);
			this.mainDialog.updateBoard();
			break;
		case FIBS_YouAcceptDouble:
			parseGameMessage(s);
			this.mainDialog.getBoard().setYouDouble(false);
			this.mainDialog.updateBoard();
			break;
		case FIBS_RollOrDouble:
			this.commandDispatcher.dispatch(CommandDispatcher.Command.ROLL_OR_DOUBLE);
			break;
		case FIBS_YouRoll:
			parseGameMessage(s);
			this.commandDispatcher.dispatch(CommandDispatcher.Command.YOUR_MOVE, "2");
			break;
		case FIBS_YourTurnToMove:	// First roll of the game, always 2 moves
			this.commandDispatcher.dispatch(CommandDispatcher.Command.YOUR_MOVE, "2");
			break;
		case FIBS_YouInvited:
			parseYouInvited(s);
			break;
		case FIBS_PleaseMove:
			parsePleaseMove(s);
			this.mainDialog.playSound(Cue.YourTurn);
			parseGameMessage(s);
			break;
		case FIBS_PlayerWantsToResign:
			parsePlayerWantsToResign(s);
			break;
		case FIBS_ATTENTION:
		case FIBS_ShuttingDown:
		case FIBS_Rebooting:
			this.mainDialog.playSound(Cue.FibsAttention);
		case FIBS_GamesWillBeSaved:
		case FIBS_UnknownCommand:
		case FIBS_GameWasSaved:
			this.commandDispatcher.writeSystemMessageln(SystemMessagesTextPane.ERROR, s);
			break;
		case FIBS_Unknown:
			this.commandDispatcher.writeSystemMessageln(SystemMessagesTextPane.ERROR, "Unknown message from FIBS: '" + s + "'");
			this.commandDispatcher.writeSystemMessageln(SystemMessagesTextPane.ERROR, "First char is " + new Integer(s.charAt(0)));
			break;
		default:
			if (this.commandDispatcher.getProperties().isDEBUG_ShowUnhandledCookies()) {
				String t = "Cookie? " + cookie + " '" + s + "'";
				this.commandDispatcher.writeSystemMessageln(SystemMessagesTextPane.ERROR, t);
				System.out.println(t);
				t = "Last message was " + lastCookie1 + " '" + lastMessage1 + "'";
				if (cookie == FIBS_Unknown) {
					this.commandDispatcher.writeSystemMessageln(SystemMessagesTextPane.ERROR, t);
					System.out.println(t);
				}
			}
			break;
		}
	}

	/** Replay a message from a saved game or a previous move in this game
	 * @param s
	 */
	public void parseReplayMessage(int cookie, String s) {
		if (DEBUG)
			System.out.println("cookie = " + cookie);
		switch (cookie) {
		case FIBS_Board:
			this.commandDispatcher.dispatch(CommandDispatcher.Command.FIBS_BOARD, s);
			break;
		default:
//			if (this.commandDispatcher.getProperties().isDEBUG_ShowUnhandledCookies())
				this.commandDispatcher.writeSystemMessageln(SystemMessagesTextPane.ERROR, "Replay? " + cookie + " '" + s + "'");
			break;
		}
	}

	private enum RegState {
		name,
		password,
		password1,
		done
	}
	private RegState regState = RegState.name;
	
	private void parseMessageRegister(String s, int fibsCookie) {
		int profileId = this.commandDispatcher.getProperties().getSelectedProfile();
		String password;
		switch (fibsCookie) {
		case FIBS_LoginPrompt:
			this.commandDispatcher.writeRegisterUserMessage("Logging in...");
			this.commandDispatcher.writeSystemMessageln(s);
			clientConnection.sendMessage("guest" + eol);
			break;
/*		case FIBS_NewUserName:
			String userName = this.commandDispatcher.getProperties().getUserName(profileId);
			this.commandDispatcher.writeRegisterUserMessage("Registering user " + userName);
			this.clientConnection.sendMessage("name " + userName + eol);
			regState = RegState.password;
			break;
*/
		case FIBS_GivePassword:
			password = this.commandDispatcher.getProperties().getPassword(profileId);
			this.clientConnection.sendMessage(password + eol);
			break;
		case FIBS_RetypePassword:
			password = this.commandDispatcher.getProperties().getPassword(profileId);
			this.clientConnection.sendMessage(password + eol);
			break;
		case FIBS_YouAreRegistered:
			this.commandDispatcher.writeRegisterUserMessage("Success!");
//			this.clientConnection.sendMessage("bye" + eol);
			this.commandDispatcher.dispatch(CommandDispatcher.Command.CONNECT_TO_SERVER);
			break;
		case FIBS_UseAnotherName:
		case FIBS_OneUserPerPerson:
			this.commandDispatcher.dispatch(CommandDispatcher.Command.BAD_NEW_USER, s);
			break;
		case CLIP_MOTD_BEGIN:
			if (regState == RegState.password) {
				password = this.commandDispatcher.getProperties().getPassword(profileId);
				this.clientConnection.sendMessage(password + eol);
				regState = RegState.password1;
				break;
			}
			if (regState == RegState.password1) {
				password = this.commandDispatcher.getProperties().getPassword(profileId);
				this.clientConnection.sendMessage(password + eol);
				regState = RegState.done;
				break;
			}
		}
	}

	private void showErrorMessage(String s) {
		this.commandDispatcher.writeSystemMessageln(SystemMessagesTextPane.ERROR, s);
	}

	private void disconnectFromServer() {
		this.commandDispatcher.dispatch(CommandDispatcher.Command.SHUTTING_DOWN);
		this.clientConnection.resetFIBSCookieMonster();
	}

	private	void parseWhoInfo(String s) {
		Player p = new Player();
		p.parsePlayer(s);
		if (this.finishedMatch != null && p.getName().equals(this.mainDialog.getBoard().getName())) {
			this.finishedMatch.setRatingAfterMatch(p.getRating());
			this.mainDialog.getDB().store(this.finishedMatch);
			this.finishedMatch = null;
			this.mainDialog.updateRatingPanel();
			this.mainDialog.getDB().updateOpponent(p);		
		}
		this.commandDispatcher.dispatch(CommandDispatcher.Command.PLAYER_CHANGED, p);
	}

	private void handleWhoEnd() {
		if (this.whoVirgin) {
			sendDelayedWhoCommand();
		} else if (this.needSavedGames) {
			sendDelayedSavedGamesCommand();
			this.needSavedGames = false;
		}
	}

	private	void parsePlayerLoggedOut(String s) {
		String[] ss = s.split(" ");
		this.commandDispatcher.dispatch(CommandDispatcher.Command.PLAYER_GONE, ss[1]);
		if (this.properties.isMsgLoginsAndOuts())
			this.commandDispatcher.writeSystemMessageln(SystemMessagesTextPane.NORMAL, ss[2] + " " + ss[3] + " " + ss[4]);
	}

	private	void parseAbortMatch(String s) {
		this.commandDispatcher.writeSystemMessageln(SystemMessagesTextPane.ERROR, s);
		this.mainDialog.playSound(Cue.MatchAborted);
		this.commandDispatcher.writeNetworkMessageln("show saved");
	}

	/** Just log the player to the messages list.
	 * Adding the player to the table comes with the CLIP_WHO_INFO message
	 * @param s The Login message
	 */
	private void parsePlayerLoggedIn(String s) {
		String[] ss = s.split(" ");
		if (this.properties.isMsgLoginsAndOuts())
			this.commandDispatcher.writeSystemMessageln(SystemMessagesTextPane.NORMAL, ss[2] + " " + ss[3] + " " + ss[4]);
	}

	private void parseOtherMatchMessage(String s) {
		if (this.properties.isMsgOtherMatchInfo())
			this.commandDispatcher.writeSystemMessageln(SystemMessagesTextPane.NORMAL, s);
	}

	private void parseInvite(String s) {
		String[] ss = s.split(" ");
		this.commandDispatcher.dispatch(CommandDispatcher.Command.INVITED, 
				ss[0], 
				ss[3].equals("resume") ? ss[3] : ss[5]);
		lastInviter = ss[0];
	}

	private void parseInviteWarning(String s) {
		if (lastInviter != null) {
			this.commandDispatcher.dispatch(CommandDispatcher.Command.INVITE_WARNING, lastInviter, s);
		}
	}

	private	void parseYouInvited(String s) {
		String[] ss = s.split(" ");
		this.commandDispatcher.dispatch(CommandDispatcher.Command.YOU_INVITED, ss[3]);
		this.commandDispatcher.writeSystemMessageln(SystemMessagesTextPane.NORMAL, s);
	}

	private	void parseDoubles(String s) {
		this.commandDispatcher.writeNetworkMessageln("board");
	}

	private	void startNewGame() {
		if (this.matchStartTime == null)
			this.matchStartTime = new Date();
		this.commandDispatcher.dispatch(CommandDispatcher.Command.START_GAME);
	}

	private void parseGameMessage(String s) {
		this.commandDispatcher.writeGameMessageln(s);
	}

	private	void parsePleaseMove(String s) {
		String[] ss = s.split(" ");
		this.commandDispatcher.dispatch(CommandDispatcher.Command.YOUR_MOVE, ss[2]);
	}

	private	void parsePlayerWantsToResign(String s) {
		String[] ss = s.split("\\.");
		ss[2] = "";
		ss[0] += ".";
		ss[1] += ".";
		this.commandDispatcher.writeGameMessageln(ss[0] + ss[1]);
		this.mainDialog.showResignInDialog(ss);
//		this.commandDispatcher.dispatch(CommandDispatcher.Command.YOUR_MOVE, ss[2]);
	}
	
	private void parseMatchOverMessage(String s, int cookie) {
		String[] ss = s.split(" ");
		this.commandDispatcher.dispatch(CommandDispatcher.Command.MATCH_OVER, ss[0], ss[6]);
		String opponentName = this.mainDialog.getBoard().getPlayerName()[1];
		int[] scores = this.mainDialog.getBoard().getMatchScore();
		if (this.mainDialog.getBoard().isYouPlaying()) {
			this.finishedMatch = new FinishedMatch();
			this.finishedMatch.setMatchPoints(Integer.parseInt(ss[3]));
			this.finishedMatch.setYourScore(scores[0]);
			this.finishedMatch.setOpponentScore(scores[1]);
			Date endDate = new Date();
			this.finishedMatch.setDate(endDate);
			this.finishedMatch.setDuration((int)((endDate.getTime()-this.matchStartTime.getTime())/1000));
			this.matchStartTime = null;
			this.mainDialog.getBoard().removeSavedMatch(opponentName);
			Player op = this.mainDialog.getDB().getPlayer(opponentName);
			if (op == null) {
				op = this.mainDialog.getPlayerTableModel().getPlayer(opponentName);
				this.mainDialog.getDB().store(op);
			}
			op.setSavedMatch(null);						// remove the icon from the player list
			this.finishedMatch.setOpponentId(op.getId());
			this.commandDispatcher.dispatch(CommandDispatcher.Command.PLAYER_CHANGED, op);

			this.mainDialog.getChatPane().maybeAddPlayer(opponentName);
			this.mainDialog.getChatPane().setupComboBoxCommand(ChatPane.Command.Tell);
		}
	}

	private void parseDoublesOnOff(boolean onoff) {
		this.commandDispatcher.dispatch(CommandDispatcher.Command.TOGGLE_DOUBLE, new Boolean(onoff));
		
	}

	private void parseReadyMessage(int cookie) {
		boolean b = false;
		if (cookie == FIBS_ReadyTrue)
			b = true;
		this.commandDispatcher.dispatch(CommandDispatcher.Command.READY_TO_PLAY, new Boolean(b));
	}

	private void parseChatMessage(String s, int cookie) {
		int i = s.indexOf(' ', 3);
		if (i == -1)
			return;
		String name = s.substring(3, i);
		String text = s.substring(i+1);
		this.mainDialog.playSound(Cue.ChatReceived);
		this.commandDispatcher.writeChatMessageln(name, cookie, text);
	}

	private void parseYouChatMessage(String s, int cookie) {
		this.mainDialog.playSound(Cue.ChatReceived);
		this.commandDispatcher.writeChatMessageln("You", cookie, s.substring(3));
	}

	private void parseWavesMessage(String s) {
		this.commandDispatcher.writeSystemMessageln(s);
	}

	private void parseResumeMatchTurn(String s) {
		String t = s.substring(6, s.length()-1);
		if (this.mainDialog.getBoard().getName().equals(t))
			this.matchResumingYourTurn = true;
	}

	private void parseResumeMatchLength(String s) {
		
	}

	private void handleAcceptRejectDouble() {
		this.commandDispatcher.dispatch(CommandDispatcher.Command.ACCEPT_OR_DECLINE_DOUBLE);
	}

	private void doAutoGreedyBearoff() {
		if (!this.mainDialog.getBoard().isYouPlaying())
			return;
		if (!this.properties.isAutoGreedyBearOff())
			return;
		if (this.mainDialog.getBoard().isGreedy())
			return;
		if (this.mainDialog.getBoard().isRace())
			this.commandDispatcher.dispatch(CommandDispatcher.Command.TOGGLE_GREEDY_BEAROFF);
	}

	/* (non-Javadoc)
	 * @see com.buckosoft.fibs.net.ClientAdapter#dispatch(java.lang.String)
	 */
	@Override
	public void dispatch(int cookie, String s) {
		this.dispatchMessage(cookie, s);
	}

	/* (non-Javadoc)
	 * @see com.buckosoft.fibs.net.ClientAdapter#writeSystemMessageln(com.buckosoft.fibs.net.ClientAdapter.MessageRoute, java.lang.String)
	 */
	@Override
	public void writeSystemMessageln(MessageRoute route, String s) {
		this.writeSystemMessage(route, s + eol);
	}

	/* (non-Javadoc)
	 * @see com.buckosoft.fibs.net.ClientAdapter#writeSystemMessage(com.buckosoft.fibs.net.ClientAdapter.MessageRoute, java.lang.String)
	 */
	@Override
	public void writeSystemMessage(MessageRoute route, String s) {
		if (route == ClientAdapter.MessageRoute.NETWORKOUT)
			this.commandDispatcher.writeSystemMessage(SystemMessagesTextPane.NETWORKOUT, s);
		else if (route == ClientAdapter.MessageRoute.DEBUG)
			this.commandDispatcher.writeSystemMessage(SystemMessagesTextPane.DEBUG, s);
		else if (route == ClientAdapter.MessageRoute.SYSTEM)
			this.commandDispatcher.writeSystemMessage(SystemMessagesTextPane.NORMAL, s);
		else if (route == ClientAdapter.MessageRoute.ERROR)
			this.commandDispatcher.writeSystemMessage(SystemMessagesTextPane.ERROR, s);
	}

	
	@Override
	public void connectionAborted() {
		this.commandDispatcher.dispatch(CommandDispatcher.Command.DISCONNECT_FROM_NETWORK);
	}


	//////////////////////////////////////////////////////////////////////////
	// Timer task which does a "who" after a couple of seconds in order to grab
	// more of the players on FIBS.
	private	Timer	myTimer = null;
	private	TimerTask	myTimerTask = null;

	private	void	sendDelayedWhoCommand() {
		if (myTimer != null) {
			return;
		}
		myTimer = new Timer();
		try {
			myTimerTask = new MyTimerTask(this);
			myTimer.schedule(myTimerTask, 2000);
		} catch (IllegalStateException e) {
			e.printStackTrace();
		}
	}

	private class MyTimerTask extends TimerTask {
		private	ClientReceiveParser crp;

		MyTimerTask(ClientReceiveParser crp) {
			super();
			this.crp = crp;
		}
		/* (non-Javadoc)
		 * @see java.util.TimerTask#run()
		 */
		@Override
		public void run() {
			this.crp.commandDispatcher.writeNetworkMessageln("who");
			this.crp.whoVirgin = false;
			this.crp.myTimer.cancel();
			this.crp.myTimer = null;
		}
	}

	//////////////////////////////////////////////////////////////////////////
	// Timer task which does a "who" after a couple of seconds in order to grab
	// more of the players on FIBS.
	private	Timer	myTimerSavedGames = null;
	private	TimerTask	myTimerTaskSavedGames = null;

	private	void	sendDelayedSavedGamesCommand() {
		if (myTimerSavedGames != null) {
			return;
		}
		myTimerSavedGames = new Timer();
		try {
			myTimerTaskSavedGames = new MyTimerTaskSavedGames(this);
			myTimerSavedGames.schedule(myTimerTaskSavedGames, 1000);
		} catch (IllegalStateException e) {
			e.printStackTrace();
		}
	}

	private class MyTimerTaskSavedGames extends TimerTask {
		private	ClientReceiveParser crp;

		MyTimerTaskSavedGames(ClientReceiveParser crp) {
			super();
			this.crp = crp;
		}
		/* (non-Javadoc)
		 * @see java.util.TimerTask#run()
		 */
		@Override
		public void run() {
			this.crp.commandDispatcher.writeNetworkMessageln("show saved");
			this.crp.myTimerSavedGames.cancel();
			this.crp.myTimerSavedGames = null;
			this.crp.myTimerTaskSavedGames.cancel();
			this.crp.myTimerTaskSavedGames = null;
		}
	}

}
