/******************************************************************************
 * AboutDialog.java - Display the license 
 * $Id: AboutDialog.java,v 1.3 2010/03/03 13:12:21 inim Exp $
 * 
 * BuckoFIBS - Backgammon by BuckoSoft
 * Copyright© 2009,2010 - Dick Balaska - BuckoSoft, Corp.
 * 
 * $Log: AboutDialog.java,v $
 * Revision 1.3  2010/03/03 13:12:21  inim
 * Replaced (c) sign in comment mangled by CVS default encoding back to UTF-8
 *
 * Revision 1.2  2010/03/03 12:19:49  inim
 * Moved source to UTF8 encoding from CP1252 encoding. To this end all source files' (c) message was updated to "Copyright© 2009,2010 - Dick Balaska - BuckoSoft, Corp.". This replaces the (c) sign to UTF8, and adds the new year 2010.
 *
 * Revision 1.1  2010/02/04 05:57:53  inim
 * Mavenized project folder layout
 *
 * Revision 1.1  2009/02/14 12:20:45  dick
 * Display the license.
 *
 */

/* 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * The Original Code is BuckoFIBS, <http://www.buckosoft.com/BuckoFIBS/>.
 * The Initial Developer of the Original Code is Dick Balaska and BuckoSoft, Corp.
 * 
 */
package com.buckosoft.fibs.BuckoFIBS.gui;

import java.awt.Dialog;
import java.awt.Font;
import java.awt.Frame;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.io.IOException;

import javax.swing.JDialog;
import javax.swing.JEditorPane;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;

/** Display the license
 * @author Dick Balaska
 * @since 2009/02/14
 * @version $Revision: 1.3 $ <br> $Date: 2010/03/03 13:12:21 $
 * @see <a href="http://cvs.buckosoft.com/Projects/BuckoFIBS/BuckoFIBS/src/com/buckosoft/fibs/BuckoFIBS/gui/AboutDialog.java">cvs AboutDialog.java</a>
 */
public class AboutDialog extends JDialog {

	private static final long serialVersionUID = 1L;
	private JPanel jContentPane = null;
	private JLabel jLabelTitle = null;
	private JEditorPane jEditorPaneLicense = null;
	private JScrollPane jScrollPaneLicense = null;
	private JLabel jLabel = null;
	private JLabel jLabelHope = null;

	/** Default constructor
	 * 
	 */
	public AboutDialog() {
		super();
		initialize();
	}

	/** Construct the AboutDialog centered over a Frame
	 * @param arg0
	 */
	public AboutDialog(Frame arg0) {
		super(arg0);
		initialize();
		this.setLocationRelativeTo(arg0);
	}

	/** Construct the AboutDialog centered over a Dialog
	 * @param arg0
	 */
	public AboutDialog(Dialog arg0) {
		super(arg0);
		initialize();
		this.setLocationRelativeTo(arg0);
	}

	/** Display the AboutDialog
	 * @param args None that i know of
	 */
	public static void main(String[] args) {
		AboutDialog aboutDialog = new AboutDialog();
		aboutDialog.setVisible(true);
	}

	/** Set the version number to display in the dialog.
	 * @param version The version as "0.1"
	 */
	public void setVersion(String version) {
		this.setTitle("About BuckoFIBS-" + version);
		this.jLabelTitle.setText("BuckoFIBS-" + version);
	}

	/**
	 * This method initializes jEditorPaneLicense	
	 * 	
	 * @return javax.swing.JEditorPane	
	 */
	private JEditorPane getJEditorPaneLicense() {
		if (jEditorPaneLicense == null) {
			jEditorPaneLicense = new JEditorPane();
			try {
				jEditorPaneLicense.setPage(getClass().getResource("/LICENSE.html"));
				jEditorPaneLicense.setEditable(false);
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
		return jEditorPaneLicense;
	}

	/**
	 * This method initializes jScrollPaneLicense	
	 * 	
	 * @return javax.swing.JScrollPane	
	 */
	private JScrollPane getJScrollPaneLicense() {
		if (jScrollPaneLicense == null) {
			jScrollPaneLicense = new JScrollPane();
			jScrollPaneLicense.setViewportView(getJEditorPaneLicense());
		}
		return jScrollPaneLicense;
	}

	/**
	 * This method initializes this
	 * 
	 * @return void
	 */
	private void initialize() {
		this.setSize(717, 330);
		this.setTitle("About BuckoFIBS-X.X");
		this.setContentPane(getJContentPane());
	}

	/**
	 * This method initializes jContentPane
	 * 
	 * @return javax.swing.JPanel
	 */
	private JPanel getJContentPane() {
		if (jContentPane == null) {
			GridBagConstraints gridBagConstraints4 = new GridBagConstraints();
			gridBagConstraints4.gridx = 0;
			gridBagConstraints4.gridy = 2;
			jLabelHope = new JLabel();
			jLabelHope.setText("It is the hope and desire of the original author that any changes or improvements you make are reflected back to the original program.");
			jLabelHope.setFont(new Font("Dialog", Font.PLAIN, 10));
			GridBagConstraints gridBagConstraints3 = new GridBagConstraints();
			gridBagConstraints3.gridx = 0;
			gridBagConstraints3.anchor = GridBagConstraints.WEST;
			gridBagConstraints3.weighty = 0.0;
			gridBagConstraints3.insets = new Insets(0, 0, 5, 0);
			gridBagConstraints3.gridy = 1;
			jLabel = new JLabel();
			jLabel.setText("Copyright© 2009 Dick Balaska & BuckoSoft, Corp.");
			GridBagConstraints gridBagConstraints2 = new GridBagConstraints();
			gridBagConstraints2.fill = GridBagConstraints.BOTH;
			gridBagConstraints2.gridy = 3;
			gridBagConstraints2.weightx = 1.0;
			gridBagConstraints2.weighty = 1.0;
			gridBagConstraints2.gridx = 0;
			GridBagConstraints gridBagConstraints = new GridBagConstraints();
			gridBagConstraints.insets = new Insets(7, 0, 0, 0);
			gridBagConstraints.gridy = 0;
			gridBagConstraints.ipadx = 0;
			gridBagConstraints.weighty = 0.0;
			gridBagConstraints.anchor = GridBagConstraints.NORTH;
			gridBagConstraints.gridx = 0;
			jLabelTitle = new JLabel();
			jLabelTitle.setText("BuckoFIBS-X.X");
			jContentPane = new JPanel();
			jContentPane.setLayout(new GridBagLayout());
			jContentPane.add(jLabelTitle, gridBagConstraints);
			jContentPane.add(getJScrollPaneLicense(), gridBagConstraints2);
			jContentPane.add(jLabel, gridBagConstraints3);
			jContentPane.add(jLabelHope, gridBagConstraints4);
		}
		return jContentPane;
	}

}  //  @jve:decl-index=0:visual-constraint="10,10"
