/******************************************************************************
 * RatingGraphConfig.java - Configure the ratings graph
 * $Id: RatingGraphConfig.java,v 1.3 2010/03/03 13:12:22 inim Exp $
 * 
 * BuckoFIBS - Backgammon by BuckoSoft
 * Copyright© 2009,2010 - Dick Balaska - BuckoSoft, Corp.
 * 
 * $Log: RatingGraphConfig.java,v $
 * Revision 1.3  2010/03/03 13:12:22  inim
 * Replaced (c) sign in comment mangled by CVS default encoding back to UTF-8
 *
 * Revision 1.2  2010/03/03 12:19:49  inim
 * Moved source to UTF8 encoding from CP1252 encoding. To this end all source files' (c) message was updated to "Copyright© 2009,2010 - Dick Balaska - BuckoSoft, Corp.". This replaces the (c) sign to UTF8, and adds the new year 2010.
 *
 * Revision 1.1  2010/02/04 05:57:53  inim
 * Mavenized project folder layout
 *
 * Revision 1.2  2010/01/29 23:10:09  dick
 * Need to be able to clone().
 *
 * Revision 1.1  2010/01/26 19:11:21  dick
 * Configure the ratings graph.
 *
 */

/* 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * The Original Code is BuckoFIBS, <http://www.buckosoft.com/BuckoFIBS/>.
 * The Initial Developer of the Original Code is Dick Balaska and BuckoSoft, Corp.
 * 
 */
package com.buckosoft.fibs.domain.config;

import java.util.Date;

/** Configure the ratings graph.
 * @author Dick Balaska
 * @since Jan 26, 2010
 * @see <a href="http://cvs.buckosoft.com/Projects/BuckoFIBS/BuckoFIBS/src/com/buckosoft/fibs/domain/config/RatingGraphConfig.java">cvs RatingGraphConfig.java</a>
 */
public class RatingGraphConfig implements Cloneable {
	public enum Type {
		displayAllMatches,
		displayLastXMatches,
		displayFromDate
	}
	
	private	Type	type = Type.displayLastXMatches;
	private	int		matchCount = 100;
	private	Date	firstMatchDate = new Date(0);
	
	public RatingGraphConfig clone() {
		RatingGraphConfig rgc = new RatingGraphConfig();
		rgc.setTypeAsInt(this.getTypeAsInt());
		rgc.setMatchCount(this.getMatchCount());
		rgc.firstMatchDate = new Date(this.firstMatchDate.getTime());
		return(rgc);
	}

	/**
	 * @return the type
	 */
	public Type getType() {
		return type;
	}

	/**
	 * @param type the type to set
	 */
	public void setType(Type type) {
		this.type = type;
	}

	public void setTypeAsInt(int t) {
		if (t == Type.displayAllMatches.ordinal())
			type = Type.displayAllMatches;
		else if (t == Type.displayLastXMatches.ordinal())
			type = Type.displayLastXMatches;
		else if (t == Type.displayFromDate.ordinal())
			type = Type.displayFromDate;
	}
	
	public int getTypeAsInt() {
		return(this.type.ordinal());
	}

	/**
	 * @return the matchCount
	 */
	public int getMatchCount() {
		return matchCount;
	}

	/**
	 * @param matchCount the matchCount to set
	 */
	public void setMatchCount(int matchCount) {
		this.matchCount = matchCount;
	}

	/**
	 * @return the firstMatchDate
	 */
	public Date getFirstMatchDate() {
		return firstMatchDate;
	}

	/**
	 * @param firstMatchDate the firstMatchDate to set
	 */
	public void setFirstMatchDate(Date firstMatchDate) {
		this.firstMatchDate = firstMatchDate;
	}
	
	
}
