/******************************************************************************
 * FIBSAttributes.java - Fetch what we need to connect to the server
 * $Id: FIBSAttributes.java,v 1.3 2010/03/03 13:12:21 inim Exp $
 * 
 * BuckoFIBS - Backgammon by BuckoSoft
 * Copyright© 2009,2010 - Dick Balaska - BuckoSoft, Corp.
 * 
 * $Log: FIBSAttributes.java,v $
 * Revision 1.3  2010/03/03 13:12:21  inim
 * Replaced (c) sign in comment mangled by CVS default encoding back to UTF-8
 *
 * Revision 1.2  2010/03/03 12:19:49  inim
 * Moved source to UTF8 encoding from CP1252 encoding. To this end all source files' (c) message was updated to "Copyright© 2009,2010 - Dick Balaska - BuckoSoft, Corp.". This replaces the (c) sign to UTF8, and adds the new year 2010.
 *
 * Revision 1.1  2010/02/04 05:57:53  inim
 * Mavenized project folder layout
 *
 * Revision 1.6  2009/02/14 12:29:56  dick
 * BuckoFIBS is released under the GNU license.
 *
 * Revision 1.5  2009/02/02 08:36:54  dick
 * Add getAppSignature().
 *
 * Revision 1.4  2009/01/28 18:15:52  dick
 * Prettier cvs link in the javadoc.
 *
 * Revision 1.3  2009/01/18 06:17:33  dick
 * Fix the cvs link.
 *
 * Revision 1.2  2009/01/18 05:17:40  dick
 * Javadoc.
 *
 * Revision 1.1  2009/01/12 07:46:06  dick
 * Fetch what we need to connect to the server.
 *
 */

/* 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * The Original Code is BuckoFIBS, <http://www.buckosoft.com/BuckoFIBS/>.
 * The Initial Developer of the Original Code is Dick Balaska and BuckoSoft, Corp.
 * 
 */
package com.buckosoft.fibs.net;

/** Fetch what we need to connect to the server.
 * @author Dick Balaska
 * @since 2009/01/11
 * @version $Revision: 1.3 $ <br> $Date: 2010/03/03 13:12:21 $
 * @see <a href="http://cvs.buckosoft.com/Projects/BuckoFIBS/BuckoFIBS/src/com/buckosoft/fibs/net/FIBSAttributes.java">cvs FIBSAttributes.java</a>
 */
public interface FIBSAttributes {

	/** Get the name of the server
	 * (probably either <code>fibs.com</code> or <code>lbd.buckosoft.com</code>)
	 * @return The name of the server
	 */
	String	getServerName();
	
	/** Get the port number that we are connecting to.
	 * @return probably 4321 
	 */
	int		getServerPort();
	
	/** Get the user name
	 * @return The user name
	 */
	String	getUserName();
	
	/** Get the user's password in clear text
	 * @return The clear text password
	 */
	String	getUserPassword();

	/** Get the client app signature, the name and version.
	 * @return Something like "BuckoFIBS_1.0"
	 */
	String	getAppSignature();

	/** Should we display messages as we send them?
	 * @return true = yes
	 */
	boolean isDisplayXmit();
	
	/** Should messages that are received be sent to stdout for logging?
	 * @return true = log messages to stdout
	 */
	boolean	isStdoutNetworkMessages();
}
